package fxGui.debugger.variables

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive

/**
 * Common interface of all reflection based properties.
 *
 * @author Konstantin Kopper
 * @since 2.0.0
 * @property T The type of the property.
 */
internal interface ReflectionProperty<T> : CoroutineScope {

    /**
     * Return the current value of the property.
     */
    fun get(): T

    /**
     * Periodically check if the underlying has changed. If so, [callback] is called.
     * After each check, 250 milliseconds elapse before the next check.
     *
     * @author Konstantin Kopper
     * @since 2.0.0
     */
    suspend fun observe(callback: () -> Unit) {
        var oldValue = get()
        while (isActive) {
            val currentValue = get()
            if (oldValue != currentValue) {
                callback()
                oldValue = currentValue
            }

            delay(250)
        }
    }
}
