package jani.interaction.basic

import jani.model.models.Model
import kotlinx.serialization.DeserializationStrategy
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.JsonContentPolymorphicSerializer
import kotlinx.serialization.json.JsonElement
import kotlinx.serialization.json.jsonObject

@Serializable(with = AnyModel.Companion::class)
sealed class AnyModel {

    // TODO check if best solution
    @Serializable
    data class JaniModel(private val m: Model) : AnyModel()

    @Serializable
    data class Textual(
        val name: String,
        val parts: List<Part>
    ) : AnyModel() {
        constructor(name: String, vararg parts: Part) : this(name, parts.asList())

        @Serializable
        data class Part(
            val name: String,
            val role: String,
            val part: String
        )
    }

    companion object : JsonContentPolymorphicSerializer<AnyModel>(AnyModel::class) {
        override fun selectDeserializer(element: JsonElement): DeserializationStrategy<out AnyModel> =
            if (element.jsonObject.containsKey("parts")) Textual.serializer() else JaniModel.serializer()
    }
}
