package jani.interaction.basic

import kotlinx.serialization.KSerializer
import kotlinx.serialization.Serializable
import kotlinx.serialization.SerializationException
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder

@Serializable(with = Extension.Companion::class)
sealed class Extension {

    object PersistentState : Extension()

    companion object : KSerializer<Extension> {
        override val descriptor = PrimitiveSerialDescriptor("jani.interaction.basic.Extension", PrimitiveKind.STRING)

        override fun deserialize(decoder: Decoder): Extension = when (val s = decoder.decodeString()) {
            "persistent-state" -> PersistentState
            else -> throw SerializationException("Unexpected extension $s")
        }

        override fun serialize(encoder: Encoder, value: Extension) = when (value) {
            is PersistentState -> encoder.encodeString("persistent-state")
        }
    }
}
