package jani.interaction.basic

import kotlinx.serialization.KSerializer
import kotlinx.serialization.Serializable
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder

@Suppress("ClassName")
@Serializable(ModellingFormalism.Companion::class)
sealed class ModellingFormalism(val string: String) {

    val asString
        get() = string

    object IOSA : ModellingFormalism("iosa")
    object Modest : ModellingFormalism("modest")
    object pGCL : ModellingFormalism("pgcl")
    object PRISM : ModellingFormalism("prism")
    object xSADF : ModellingFormalism("xsadf")

    private class ModellingFormalismImpl(string: String) : ModellingFormalism(string) {
        init {
            require(string.startsWith("x-")) { "The name of custom modelling formalisms must start with 'x-'." }
        }

        override fun toString(): String = asString
    }

    override fun equals(other: Any?) = other != null && other is ModellingFormalism && other.string == this.string

    override fun hashCode() = string.hashCode()

    companion object : KSerializer<ModellingFormalism> {
        override val descriptor: SerialDescriptor =
            PrimitiveSerialDescriptor("jani.interaction.basic.ModellingFormalism", PrimitiveKind.STRING)

        override fun deserialize(decoder: Decoder): ModellingFormalism = when (val s = decoder.decodeString()) {
            "modest" -> Modest // Workaround: The value in the map below is somehow 'null'...
            in stringMap -> stringMap[s]!!
            else -> new(s)
        }

        override fun serialize(encoder: Encoder, value: ModellingFormalism) = encoder.encodeString(value.asString)

        fun new(s: String): ModellingFormalism = ModellingFormalismImpl(s)

        private val stringMap =
            mapOf("iosa" to IOSA, "modest" to Modest, "pgcl" to pGCL, "prism" to PRISM, "xsadf" to xSADF)
    }
}
