package jani.interaction.basic

import kotlinx.serialization.KSerializer
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.SerializationException
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.descriptors.buildClassSerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import kotlinx.serialization.json.JsonDecoder
import kotlinx.serialization.json.JsonPrimitive
import kotlinx.serialization.json.booleanOrNull
import kotlinx.serialization.json.intOrNull

@Serializable
data class ParameterValue(
    @SerialName("id") val identifier: Identifier,
    val value: Values
) {

    @Serializable(with = Values.Companion::class)
    sealed class Values {
        object True : Values()
        object False : Values()

        data class Number(val number: Int) : Values()
        data class String(val string: kotlin.String) : Values()

        companion object : KSerializer<Values> {
            override val descriptor: SerialDescriptor =
                buildClassSerialDescriptor("jani.interaction.basic.ParameterValue.Value")

            override fun deserialize(decoder: Decoder): Values {
                val jsonDecoder =
                    decoder as? JsonDecoder ?: throw SerializationException("Only JSON can be deserialized.")
                val p = jsonDecoder.decodeJsonElement() as? JsonPrimitive
                    ?: throw SerializationException("Expected a JSON primitive.")
                return p.booleanOrNull?.let { if (it) True else False }
                    ?: p.intOrNull?.let { Number(it) }
                    ?: String(p.content)
            }

            override fun serialize(encoder: Encoder, value: Values) {
                when (value) {
                    is True -> encoder.encodeBoolean(true)
                    is False -> encoder.encodeBoolean(false)
                    is Number -> encoder.encodeInt(value.number)
                    is String -> encoder.encodeString(value.string)
                }
            }
        }
    }
}
