package jani.model

import kotlinx.serialization.KSerializer
import kotlinx.serialization.Serializable
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder

@Serializable(with = Identifier.Companion::class)
data class Identifier(private val identifier: String) {

    init {
        assert(identifier.matches(Regex("[^#].*$")))
    }

    val plainString: String
        get() = identifier

    override fun toString(): String = identifier

    companion object : KSerializer<Identifier> {
        override val descriptor: SerialDescriptor =
            PrimitiveSerialDescriptor("jani.model.Identifier", PrimitiveKind.STRING)

        override fun deserialize(decoder: Decoder): Identifier = Identifier(decoder.decodeString())

        override fun serialize(encoder: Encoder, value: Identifier) = encoder.encodeString(value.identifier)
    }
}
