package jani.model.models

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable

@Serializable
enum class ModelType {

    /**
     * A labelled transition system (or Kripke structure or finite state automaton) (untimed).
     */
    @SerialName("lts")
    LTS,

    /**
     * A discrete-time Markov chain (untimed)
     */
    @SerialName("dtmc")
    DTMC,

    /**
     * A continuous-time Markov chain (timed)
     */
    @SerialName("ctmc")
    CTMC,

    /**
     * A discrete-time Markov decision process (untimed)
     */
    @SerialName("mdp")
    MDP,

    /**
     * A continuous-time Markov decision process (timed)
     */
    @SerialName("ctmdp")
    CTMDP,

    /**
     * A Markov automaton (timed)
     */
    @SerialName("ma")
    MA,

    /**
     * A timed automaton (timed)
     */
    @SerialName("ta")
    TA,

    /**
     * A probabilistic timed automaton (timed)
     */
    @SerialName("pta")
    PTA,

    /**
     * A stochastic timed automaton (timed)
     */
    @SerialName("sta")
    STA,

    /**
     * A hybrid automaton (timed)
     */
    @SerialName("ha")
    HA,

    /**
     * A probabilistic hybrid automaton (timed)
     */
    @SerialName("pha")
    PHA,

    /**
     * A stochastic hybrid automaton (timed)
     */
    @SerialName("sha")
    SHA,
}
