@file:UseSerializers(DateSerializer::class)

package org.gitlab.api.v4.commits

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.UseSerializers
import util.DateSerializer
import java.util.Date

/**
 * A commit as returned by the GitLab API v4.
 *
 * @author Konstantin Kopper
 * @since 2.1.0
 * @property id The full commit SHA.
 * @property shortId A shortened commit SHA.
 * @property title Seems to be the same as [message].
 * @property authorName TODO
 * @property authorEmail TODO
 * @property authoredDate TODO
 * @property committerName TODO
 * @property committerEmail TODO
 * @property committedDate TODO
 * @property createdAt TODO
 * @property message The commit message.
 * @property parentIds A list containing the commit SHAs of the parent commits.
 */
@Serializable
data class Commit(
    val id: String,
    @SerialName("short_id") val shortId: String,
    val title: String,
    @SerialName("author_name") val authorName: String,
    @SerialName("author_email") val authorEmail: String,
    @SerialName("authored_date") val authoredDate: Date,
    @SerialName("committer_name") val committerName: String,
    @SerialName("committer_email") val committerEmail: String,
    @SerialName("committed_date") val committedDate: Date,
    @SerialName("created_at") val createdAt: Date,
    val message: String,
    @SerialName("parent_ids") val parentIds: List<String>,
) {

    init {
        assert(id.startsWith(shortId))
    }
}
