package jani.interaction.basic

import kotlinx.serialization.json.Json
import kotlinx.serialization.json.JsonPrimitive
import kotlinx.serialization.json.buildJsonArray
import kotlinx.serialization.json.buildJsonObject
import kotlinx.serialization.json.decodeFromJsonElement
import kotlinx.serialization.json.encodeToJsonElement
import kotlinx.serialization.json.jsonArray
import kotlinx.serialization.json.jsonObject
import kotlinx.serialization.json.jsonPrimitive
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertTrue
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertAll
import org.junit.jupiter.api.assertDoesNotThrow

@DisplayName("jani.interaction.basic.AnyModel serialization")
internal class AnyModelSerializationTest {

    @Test
    @DisplayName("Serialize 'Textual' 1")
    fun testSerializeTextual1() {
        val model: AnyModel = AnyModel.Textual(
            "helloWorld",
            AnyModel.Textual.Part("helloWorld.pseuco", "source", "mainAgent {\n    println(\"Hello World\");\n}")
        )

        val msg = assertDoesNotThrow { Json.encodeToJsonElement(model) }

        val o = assertDoesNotThrow { msg.jsonObject }

        assertAll(
            {
                assertTrue { "name" in o }
                assertEquals("helloWorld", o["name"]!!.jsonPrimitive.content)
            },
            {
                assertTrue { "parts" in o }
                val parts = assertDoesNotThrow { o["parts"]!!.jsonArray }
                assertEquals(1, parts.size)
                val part = assertDoesNotThrow { parts.first().jsonObject }
                assertAll(
                    {
                        assertTrue { "name" in part }
                        assertEquals("helloWorld.pseuco", part["name"]!!.jsonPrimitive.content)
                    },
                    {
                        assertTrue { "role" in part }
                        assertEquals("source", part["role"]!!.jsonPrimitive.content)
                    },
                    {
                        assertTrue { "part" in part }
                        assertEquals(
                            "mainAgent {\n    println(\"Hello World\");\n}",
                            part["part"]!!.jsonPrimitive.content
                        )
                    }
                )
            }
        )
    }

    @Test
    @DisplayName("Serialize 'Textual' 2")
    fun testSerializeTextual2() {
        val model = AnyModel.Textual(
            "helloWorld",
            AnyModel.Textual.Part("helloWorld.pseuco", "source", "mainAgent {\n    println(\"Hello World\");\n}")
        )

        val msg = assertDoesNotThrow { Json.encodeToJsonElement<AnyModel>(model) }

        val o = assertDoesNotThrow { msg.jsonObject }

        assertAll(
            {
                assertTrue { "name" in o }
                assertEquals("helloWorld", o["name"]!!.jsonPrimitive.content)
            },
            {
                assertTrue { "parts" in o }
                val parts = assertDoesNotThrow { o["parts"]!!.jsonArray }
                assertEquals(1, parts.size)
                val part = assertDoesNotThrow { parts.first().jsonObject }
                assertAll(
                    {
                        assertTrue { "name" in part }
                        assertEquals("helloWorld.pseuco", part["name"]!!.jsonPrimitive.content)
                    },
                    {
                        assertTrue { "role" in part }
                        assertEquals("source", part["role"]!!.jsonPrimitive.content)
                    },
                    {
                        assertTrue { "part" in part }
                        assertEquals(
                            "mainAgent {\n    println(\"Hello World\");\n}",
                            part["part"]!!.jsonPrimitive.content
                        )
                    }
                )
            }
        )
    }

    @Test
    @DisplayName("Deserialize 'Textual'")
    fun testDeserializeTextual() {
        val msg = buildJsonObject {
            put("name", JsonPrimitive("helloWorld"))
            put("parts", buildJsonArray {
                add(buildJsonObject {
                    put("name", JsonPrimitive("helloWorld.pseuco"))
                    put("role", JsonPrimitive("source"))
                    put("part", JsonPrimitive("mainAgent {\n    println(\"Hello World\");\n}"))
                })
            })
        }

        val model = assertDoesNotThrow { Json.decodeFromJsonElement<AnyModel>(msg) }

        assertTrue { model is AnyModel.Textual }
        model as AnyModel.Textual

        assertAll(
            {
                assertEquals("helloWorld", model.name)
            },
            {
                assertEquals(1, model.parts.size)
                val part = model.parts.first()
                assertAll(
                    { assertEquals("helloWorld.pseuco", part.name) },
                    { assertEquals("source", part.role) },
                    { assertEquals("mainAgent {\n    println(\"Hello World\");\n}", part.part) }
                )
            }
        )
    }
}
