package jani.interaction.basic

import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import kotlinx.serialization.json.JsonPrimitive
import kotlinx.serialization.json.decodeFromJsonElement
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertTrue
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

@DisplayName("jani.interaction.basic.ModellingFormalism serialization")
internal class ModellingFormalismSerializationTest {

    @Test
    @DisplayName("Serialize 'IOSA' 1")
    fun testIOSASerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.IOSA
        val msg = Json.encodeToString(formalism)

        assertEquals("\"iosa\"", msg)
    }

    @Test
    @DisplayName("Serialize 'IOSA' 2")
    fun testIOSASerialization2() {
        val formalism = ModellingFormalism.IOSA
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"iosa\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'IOSA'")
    fun testIOSADeserialization() {
        val msg = JsonPrimitive("iosa")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism is ModellingFormalism.IOSA)
    }

    @Test
    @DisplayName("Serialize 'Modest' 1")
    fun testModestSerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.Modest
        val msg = Json.encodeToString(formalism)

        assertEquals("\"modest\"", msg)
    }

    @Test
    @DisplayName("Serialize 'Modest' 2")
    fun testModestSerialization2() {
        val formalism = ModellingFormalism.Modest
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"modest\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'Modest'")
    fun testModestDeserialization() {
        val msg = JsonPrimitive("modest")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism is ModellingFormalism.Modest)
    }

    @Test
    @DisplayName("Serialize 'pGCL' 1")
    fun testpGCLSerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.pGCL
        val msg = Json.encodeToString(formalism)

        assertEquals("\"pgcl\"", msg)
    }

    @Test
    @DisplayName("Serialize 'pGCL' 2")
    fun testpGCLSerialization2() {
        val formalism = ModellingFormalism.pGCL
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"pgcl\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'pGCL'")
    fun testpGCLDeserialization() {
        val msg = JsonPrimitive("pgcl")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism is ModellingFormalism.pGCL)
    }

    @Test
    @DisplayName("Serialize 'PRISM' 1")
    fun testPRISMSerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.PRISM
        val msg = Json.encodeToString(formalism)

        assertEquals("\"prism\"", msg)
    }

    @Test
    @DisplayName("Serialize 'PRISM' 2")
    fun testPRISMSerialization2() {
        val formalism = ModellingFormalism.PRISM
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"prism\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'PRISM'")
    fun testPRISMDeserialization() {
        val msg = JsonPrimitive("prism")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism is ModellingFormalism.PRISM)
    }

    @Test
    @DisplayName("Serialize 'xSADF' 1")
    fun testxSADFSerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.xSADF
        val msg = Json.encodeToString(formalism)

        assertEquals("\"xsadf\"", msg)
    }

    @Test
    @DisplayName("Serialize 'xSADF' 2")
    fun testxSADFSerialization2() {
        val formalism = ModellingFormalism.xSADF
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"xsadf\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'xSADF'")
    fun testxSADFDeserialization() {
        val msg = JsonPrimitive("xsadf")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism is ModellingFormalism.xSADF)
    }

    @Test
    @DisplayName("Serialize custom 1")
    fun testCustomSerialization1() {
        val formalism: ModellingFormalism = ModellingFormalism.new("x-pseuco")
        val msg = Json.encodeToString(formalism)

        assertEquals("\"x-pseuco\"", msg)
    }

    @Test
    @DisplayName("Serialize custom 2")
    fun testCustomSerialization2() {
        val formalism = ModellingFormalism.new("x-pseuco")
        val msg = Json.encodeToString<ModellingFormalism>(formalism)

        assertEquals("\"x-pseuco\"", msg)
    }

    @Test
    @DisplayName("Deserialize custom")
    fun testCustomDeserialization() {
        val msg = JsonPrimitive("x-pseuco")
        val formalism = Json.decodeFromJsonElement<ModellingFormalism>(msg)

        assertTrue(formalism == ModellingFormalism.new("x-pseuco"))
    }
}
