package jani.model.models

import kotlinx.serialization.decodeFromString
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertTrue
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

@DisplayName("jani.model.models.ModelFeature serialization")
internal class ModelFeatureSerializationTest {

    @Test
    @DisplayName("Serialize 'Arrays' 1")
    fun testArraysSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.Arrays
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"arrays\"", msg)
    }

    @Test
    @DisplayName("Serialize 'Arrays' 2")
    fun testArraysSerialization2() {
        val modelFeature = ModelFeature.Arrays
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"arrays\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'Arrays'")
    fun testArraysDeserialization() {
        val msg = "\"arrays\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.Arrays }
    }

    @Test
    @DisplayName("Serialize 'Datatypes' 1")
    fun testDatatypesSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.Datatypes
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"datatypes\"", msg)
    }

    @Test
    @DisplayName("Serialize 'Datatypes' 2")
    fun testDatatypesSerialization2() {
        val modelFeature = ModelFeature.Datatypes
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"datatypes\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'Datatypes'")
    fun testDatatypesDeserialization() {
        val msg = "\"datatypes\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.Datatypes }
    }

    @Test
    @DisplayName("Serialize 'DerivedOperators' 1")
    fun testDerivedOperatorsSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.DerivedOperators
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"derived-operators\"", msg)
    }

    @Test
    @DisplayName("Serialize 'DerivedOperators' 2")
    fun testDerivedOperatorsSerialization2() {
        val modelFeature = ModelFeature.DerivedOperators
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"derived-operators\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'DerivedOperators'")
    fun testDerivedOperatorsDeserialization() {
        val msg = "\"derived-operators\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.DerivedOperators }
    }

    @Test
    @DisplayName("Serialize 'EdgePriorities' 1")
    fun testEdgePrioritiesSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.EdgePriorities
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"edge-priorities\"", msg)
    }

    @Test
    @DisplayName("Serialize 'EdgePriorities' 2")
    fun testEdgePrioritiesSerialization2() {
        val modelFeature = ModelFeature.EdgePriorities
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"edge-priorities\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'EdgePriorities'")
    fun testEdgePrioritiesDeserialization() {
        val msg = "\"edge-priorities\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.EdgePriorities }
    }

    @Test
    @DisplayName("Serialize 'Functions' 1")
    fun testFunctionsSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.Functions
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"functions\"", msg)
    }

    @Test
    @DisplayName("Serialize 'Functions' 2")
    fun testFunctionsSerialization2() {
        val modelFeature = ModelFeature.Functions
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"functions\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'Functions'")
    fun testFunctionsDeserialization() {
        val msg = "\"functions\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.Functions }
    }

    @Test
    @DisplayName("Serialize 'HyperbolicFunctions' 1")
    fun testHyperbolicFunctionsSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.HyperbolicFunctions
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"hyperbolic-functions\"", msg)
    }

    @Test
    @DisplayName("Serialize 'HyperbolicFunctions' 2")
    fun testHyperbolicFunctionsSerialization2() {
        val modelFeature = ModelFeature.HyperbolicFunctions
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"hyperbolic-functions\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'HyperbolicFunctions'")
    fun testHyperbolicFunctionsDeserialization() {
        val msg = "\"hyperbolic-functions\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.HyperbolicFunctions }
    }

    @Test
    @DisplayName("Serialize 'NamedExpressions' 1")
    fun testNamedExpressionsSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.NamedExpressions
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"named-expressions\"", msg)
    }

    @Test
    @DisplayName("Serialize 'NamedExpressions' 2")
    fun testNamedExpressionsSerialization2() {
        val modelFeature = ModelFeature.NamedExpressions
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"named-expressions\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'NamedExpressions'")
    fun testNamedExpressionsDeserialization() {
        val msg = "\"named-expressions\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.NamedExpressions }
    }

    @Test
    @DisplayName("Serialize 'NonDetSelection' 1")
    fun testNonDetSelectionSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.NonDetSelection
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"nondet-selection\"", msg)
    }

    @Test
    @DisplayName("Serialize 'NonDetSelection' 2")
    fun testNonDetSelectionSerialization2() {
        val modelFeature = ModelFeature.NonDetSelection
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"nondet-selection\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'NonDetSelection'")
    fun testNonDetSelectionDeserialization() {
        val msg = "\"nondet-selection\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.NonDetSelection }
    }

    @Test
    @DisplayName("Serialize 'TradeoffProperties' 1")
    fun testTradeoffPropertiesSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.TradeoffProperties
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"tradeoff-properties\"", msg)
    }

    @Test
    @DisplayName("Serialize 'TradeoffProperties' 2")
    fun testTradeoffPropertiesSerialization2() {
        val modelFeature = ModelFeature.TradeoffProperties
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"tradeoff-properties\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'TradeoffProperties'")
    fun testTradeoffPropertiesDeserialization() {
        val msg = "\"tradeoff-properties\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.TradeoffProperties }
    }

    @Test
    @DisplayName("Serialize 'TrigonometricFunctions' 1")
    fun testTrigonometricFunctionsSerialization1() {
        val modelFeature: ModelFeature = ModelFeature.TrigonometricFunctions
        val msg = Json.encodeToString(modelFeature)

        assertEquals("\"trigonometric-functions\"", msg)
    }

    @Test
    @DisplayName("Serialize 'TrigonometricFunctions' 2")
    fun testTrigonometricFunctionsSerialization2() {
        val modelFeature = ModelFeature.TrigonometricFunctions
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"trigonometric-functions\"", msg)
    }

    @Test
    @DisplayName("Deserialize 'TrigonometricFunctions'")
    fun testTrigonometricFunctionsDeserialization() {
        val msg = "\"trigonometric-functions\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertTrue { modelFeature is ModelFeature.TrigonometricFunctions }
    }

    @Test
    @DisplayName("Serialize custom")
    fun testCustomSerialization2() {
        val modelFeature = ModelFeature.new("x-pseuco")
        val msg = Json.encodeToString<ModelFeature>(modelFeature)

        assertEquals("\"x-pseuco\"", msg)
    }

    @Test
    @DisplayName("Deserialize custom")
    fun testCustomDeserialization() {
        val msg = "\"x-pseuco\""
        val modelFeature = Json.decodeFromString<ModelFeature>(msg)

        assertEquals("x-pseuco", modelFeature.name.toString())
    }
}
